//simple http client

package net.binzume.android.nicoplayer.api;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.ProtocolException;
import org.apache.http.client.RedirectHandler;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.cookie.Cookie;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.DefaultRedirectHandler;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HttpContext;

import android.util.Log;

public class HttpClient {
    protected DefaultHttpClient client;
    protected List<Cookie> cookies;
    protected String content;
    protected String location;

    public HttpClient(){
        client = new DefaultHttpClient();
    }

    public boolean post(String url, String data){
        try {
            return post(url,new StringEntity(data,"UTF-8"),true);
        } catch (Exception e) {
            return false;
        }
    }

    public boolean post(String url, Params params){
        return post(url,params,true);
    }
    public boolean post(String url, Params params, boolean needContent){
        try {
            return post(url,new UrlEncodedFormEntity(params),needContent);
        } catch (Exception e) {
            return false;
        }
    }
    
    private boolean post(String url, StringEntity data, boolean needContent){
        content = null;
        try {
            HttpPost request = new HttpPost(url);
            request.setEntity(data);
            HttpResponse response = client.execute(request);
            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
                return false;
            }
            if ( needContent ) read(response);
        } catch (Exception e) {
            // debug
            for (int i=0;i<e.getStackTrace().length;i++) {
                Log.d("nicop_HTTP","Exception:"+e.getStackTrace()[i]);
            }
            return false;
        }
        
        return true;
    }

    public boolean get(String url){
        content = null;
        try {
            HttpGet request = new HttpGet(url);
            HttpResponse response = client.execute(request);
            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
                return false;
            }
            read(response);
        } catch (Exception e) {
            return false;
        }
        
        
        return true;
    }
    
    public boolean head(String url){
        content = null;
//        RedirectHandler d = client.getRedirectHandler();
        client.setRedirectHandler(new DefaultRedirectHandler() {
            public URI getLocationURI(HttpResponse res, HttpContext arg1)
                    throws ProtocolException {
                if (res.getFirstHeader("Location") != null)
                    location = res.getFirstHeader("Location").getValue();
                return super.getLocationURI(res,arg1);
            }
        });

        try {
            HttpHead request = new HttpHead(url);
            HttpResponse response;
            response = client.execute(request);
//            client.setRedirectHandler(d);
//            if (response.getFirstHeader("Location") != null)
//                location = response.getFirstHeader("Location").getValue();
            if (response.getStatusLine().getStatusCode() == HttpStatus.SC_BAD_REQUEST) {
                return false;
            }
        } catch (Exception e) {
            return false;
        }
        
        return true;
    }
    
    private boolean read(HttpResponse res) throws IOException {
        HttpEntity ent = res.getEntity();
        InputStream inputStream = ent.getContent();
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));

        content = "";
        char buf[] = new char[10240];
        int size;
        while ((size=reader.read(buf,0,buf.length)) >= 0) {
            content+=String.valueOf(buf,0,size);
        }/*
        content = "";
        String line;
        while ((line = reader.readLine()) != null) {
            content += line;
        }*/

        return true;
    }

    public List<Cookie> getCookies(){
        List<Cookie> cookies = client.getCookieStore().getCookies();
        return cookies;
    }

    public void setCookie(Cookie cookie){
        client.getCookieStore().addCookie(cookie);
    }
    
    public String getContent(){
        return content;
    }
    
    public String getLocation(){
        return location;
    }

    public static class Params extends ArrayList<NameValuePair>{
        private static final long serialVersionUID = 2525123L;

        public void put(String key, String value) {
            add(new BasicNameValuePair(key,value));
        }
    }
    
}
