/**
 * jRjR }CXg API
 * 
 * @author binzume <kawahira__binzume.net>
 */
package net.binzume.android.nicoplayer.api;

import org.json.*;

import android.util.Log;
import java.util.List;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MyListAPI {
    public static final String API_URL = "http://www.nicovideo.jp/api/";
    public static final String MYLIST_URL = "http://www.nicovideo.jp/my/mylist";
    
    /**
     * }CXg̈ꗗ擾
     * 
     * sNULL,΋List
     * }CXg̓ê͎擾Ȃ
     * 
     * @param session ZbV
     * @return
     */
    public static List<MyList> getLists(NicoSession session){
        HttpClient client = new HttpClient();
        client.setCookie(session.cookie);

        if(!client.get(API_URL + "mylistgroup/list")){
            return null;
        }

        String data = client.getContent();
        if(data.length() <= 0){
            return null;
        }
 
        ArrayList<MyList> list;
        try {
            JSONObject json = new JSONObject(data);
            JSONArray jslist = json.getJSONArray("mylistgroup");
            list = new ArrayList<MyList>();
            for (int i=0;i<jslist.length();i++){
                JSONObject jitem = jslist.getJSONObject(i);
                MyList item = new MyList();
                item.id = jitem.getInt("id");
                item.userId = jitem.getInt("user_id");
                item.name = jitem.getString("name");
                item.isPublic = jitem.getInt("public");
                list.add(item);
            }
        } catch (Exception e) {
            return null;
        }
        
        
        return list;
    }

    public static MyList getDefault(NicoSession session){
        HttpClient client = new HttpClient();
        client.setCookie(session.cookie);

        if(!client.get(API_URL + "deflist/list")){
            Log.d("nicoplayer_api","err1");
            return null;
        }

        String data = client.getContent();
        if(data.length() <= 0){
            Log.d("nicoplayer_api","err2");
            return null;
        }
 
        MyList list = null;
        try {
            JSONObject json = new JSONObject(data);
            if (!json.getString("status").equals("ok")){
                Log.d("nicoplayer_api","status:"+json.getString("status"));
                return null;
            }

            list = new MyList();
            list.id = -1;
            list.userId = 0;
            list.name = "Ƃ肠";
            list.isPublic = 0;
            list.items = new ArrayList<MyList.Item>();
            
            JSONArray jslist = json.getJSONArray("mylistitem");
            for (int i=0;i<jslist.length();i++){
                JSONObject jitem = jslist.getJSONObject(i);
                MyList.Item item = new MyList.Item();
                item.type = jitem.getInt("item_type");
                item.id = jitem.getInt("item_id");
                item.description = jitem.getString("description");
                JSONObject itemdata = jitem.getJSONObject("item_data");
                item.videoId = itemdata.getString("video_id");
                item.title = itemdata.getString("title");
                item.thumbnailUrl = itemdata.getString("thumbnail_url");
                item.lastResBody = itemdata.getString("last_res_body");

                list.items.add(item);
            }
        } catch (Exception e) {
            return null;
        }
        return list;
    }
    public static boolean addDefault(NicoSession session, String vid){
        HttpClient client = new HttpClient();
        client.setCookie(session.cookie);
        Log.d("nicop","addDefault "+vid);

        if(!client.get(VideoAPI.WATCH_URL+ vid)){
            return false;
        }
        String data = client.getContent();
        Log.d("nicop","addDefault "+vid);

        Pattern pattern = Pattern.compile("\"addVideoToDeflist\\((\\d+),\\s*[\"\']([^\"\']+)[\"\']\\s*\\);");
        Matcher matcher = pattern.matcher(data);
        Log.d("nicop","addDefault "+vid);
        if (matcher.find()){
            Log.d("nicop",matcher.group(1));
            Log.d("nicop",matcher.group(2));
            HttpClient.Params params = new HttpClient.Params();
            params.put("item_id", matcher.group(1));
            params.put("token", matcher.group(2));

            if(!client.post(API_URL+"deflist/add", params,false)){
                return false;
            }
        }

        return true;
    }
    
    public static MyList get(NicoSession session, int id) {
        MyList mylist = new MyList();
        mylist.id = id;
        mylist.userId = 0;
        mylist.name = "Ƃ肠";
        mylist.isPublic = 0;
        return get(session, mylist);
    }

    public static MyList get(NicoSession session, MyList mylist) {
        if (mylist.id<0) return getDefault(session);
        HttpClient client = new HttpClient();
        client.setCookie(session.cookie);

        if(!client.get(API_URL + "mylist/list?group_id=" + mylist.id)){
            Log.d("nicoplayer_api","err1");
            return null;
        }

        String data = client.getContent();
        if(data.length() <= 0){
            Log.d("nicoplayer_api","err2");
            return null;
        }

        try {
            JSONObject json = new JSONObject(data);
            if (!json.getString("status").equals("ok")){
                Log.d("nicoplayer_api","status:"+json.getString("status"));
                return null;
            }

            
            mylist.items = new ArrayList<MyList.Item>();
            JSONArray jslist = json.getJSONArray("mylistitem");
            for (int i=0;i<jslist.length();i++){
                JSONObject jitem = jslist.getJSONObject(i);
                MyList.Item item = new MyList.Item();
                item.type = jitem.getInt("item_type");
                item.id = jitem.getInt("item_id");
                item.description = jitem.getString("description");
                JSONObject itemdata = jitem.getJSONObject("item_data");
                item.videoId = itemdata.getString("video_id");
                item.title = itemdata.getString("title");
                item.thumbnailUrl = itemdata.getString("thumbnail_url");
                item.lastResBody = itemdata.getString("last_res_body");

                mylist.items.add(item);
            }
        } catch (Exception e) {
            return null;
        }

        return mylist;
    }

    public static boolean remove(NicoSession session, MyList mylist, long id){
        HttpClient client = new HttpClient();
        client.setCookie(session.cookie);
        String token = getToken(client);
        if (token == null) {
            return false;
        }
        if (mylist == null || mylist.id==-1) {
            HttpClient.Params params = new HttpClient.Params();
            params.put("id_list[0][]", ""+id);
            params.put("token", token);

            //String postdata = "id_list[0][]="+id+"&token="+token;
            if(!client.post(API_URL+"deflist/delete", params)){
                //Log.d("nicoplayer_api","remove err1 "+postdata);
                return false;
            }
//            Log.d("nicoplayer_api","remove  "+postdata);
        } else {
            HttpClient.Params params = new HttpClient.Params();
            params.put("group_id", ""+mylist.id);
            params.put("id_list[0][]", ""+id);
            params.put("token", token);
            if(!client.post(API_URL+"mylist/delete", params)){
                return false;
            }
        }
        String data = client.getContent();
        
        JSONObject json;
        try {
            json = new JSONObject(data);
            if (!json.getString("status").equals("ok")){
                Log.d("nicoplayer_api","status:"+json.getString("status"));
                return false;
            }
        } catch (JSONException e) {
            return false;
        }

        return true;
    }
    
    private static String getToken(HttpClient client){
        if(!client.get(MYLIST_URL)){
            Log.d("nicoplayer_api","getToken() err1");
            return null;
        }
        String data = client.getContent();
        Pattern pattern = Pattern.compile("NicoAPI.token\\s*=\\s*\"([0-9a-f-]+)\";");
        Matcher matcher = pattern.matcher(data);
        if (matcher.find()) {
            Log.d("nicoplayer_api","getToken() : "+matcher.group(1));
            return matcher.group(1);
        }

        return null;
    }
}
