//
// LbVvLV
//
package net.binzume.android.nicoplayer.proxy;
import java.lang.InterruptedException;
import android.net.Uri;
import android.util.Log;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;  
import java.io.OutputStreamWriter;
import java.net.ServerSocket;  
import java.net.Socket;  
import java.net.URLDecoder;
import java.io.BufferedOutputStream;
import java.io.BufferedInputStream;

public class SmilevideoCacheServer  extends Thread {
    
    static final String cache_dir = "/net.binzume.android.nicoplayer";
    
    private SmilevideoCache cache;

    static final int DEFAULT_PORT = 25251;
    
    public SmilevideoCacheServer(SmilevideoCache cache) {
        this.cache = cache;
    }

    public void setCache(SmilevideoCache cache) {
        this.cache = cache;
    }

    
    @Override
    public void run() {
        Log.d("nicoplayer_SmileVideoProxy","s_start");
        
        ServerSocket server = null;
        try {
            server = new ServerSocket(DEFAULT_PORT);
            for (int i=0;;i++) {
                // ڑ󂯕t܂ő҂
                Socket client = server.accept();
                Log.d("nicoplayer_SmileVideoProxy","s_accept");
                ConnectionThread ct = new ConnectionThread(client, i);
                ct.start();
                sleep(0);
            }
        } catch (InterruptedException e) {
            Log.d("nicoplayer_SmileVideoProxy","s_stop");
        } catch (Exception e) {
            try {
                if (server != null) server.close();
            } catch (Exception ee) {
            }
        }
        Log.d("nicoplayer_SmileVideoProxy","s_finish");
    }
    
    static class DownloadThread extends Thread {
        private String url;
        private String cookie;
        private int start;
        private SmilevideoCache cache;
        public DownloadThread(SmilevideoCache cache, String url, String cookie) {
            this.url = url;
            this.cache = cache;
            this.cookie = cookie;
            this.start = cache.getCacheSize();
        }
        
        private String readline(InputStream in) throws IOException{
            byte b[] = new byte[1024];
            int pos=0;
            while(in.read(b,pos,1)>=0) {
                if (b[pos]==0x0A) break;
                pos++;
            }
            if (pos>0 && b[pos-1]==0x0D) pos--;
            return new String(b,0,pos);
        }
        
        @Override
        public void run() {
            InputStream sockin = null;
//            BufferedReader reader = null;
            BufferedWriter sockout = null;
            try {
                // ڑ
                Uri uri = Uri.parse(url);
                Socket soc = new Socket(uri.getHost(),uri.getPort()<0?80:uri.getPort());
                Log.d("nicoplayer_SmileVideoProxy","socket ok " + url);

                sockin = soc.getInputStream();
//                reader = new BufferedReader(new InputStreamReader(sockin));
                sockout = new BufferedWriter(new OutputStreamWriter(soc.getOutputStream()));
                sockout.write("GET " + uri.getPath()+"?"+uri.getQuery() + " HTTP/1.1\r\n");

                sockout.write("Host: " + uri.getHost() + "\r\n");
                sockout.write("Cookie: " + cookie + "\r\n");
                sockout.write("Connection: close\r\n");
                if (start>0) {
                    sockout.write("Range: bytes=" + start + "-\r\n");
                    Log.d("nicop_SmileVideoProxy","response start " + start+"-");
                }
                sockout.write("\r\n");
                sockout.flush();
                Log.d("nicop_SmileVideoProxy","request ok " + cookie + "  s:"+start);
                Log.d("nicop_SmileVideoProxy","GET " + uri.getPath()+"?"+uri.getQuery() + " HTTP/1.1\r\n");
                Log.d("nicop_SmileVideoProxy","Host: " + uri.getHost() + "\r\n");
                Log.d("nicop_SmileVideoProxy","Cookie: " + cookie + "\r\n");

                String line;
                int size=-1;
                int content_length=0;
                String type = "";
//                while ((line = reader.readLine()) != null) {                  
                while ((line = readline(sockin)) != null) {                  
                    Log.d("nicop_SmileVideoProxy"," header " + line);
                    if (line.length()==0) break;
                    
                    if (line.startsWith("Content-Length:")) {
                        String h[] = line.split(":\\s*");
                        if (h.length>1) {
                            content_length = Integer.parseInt(h[1]);
                            if (start==0) {
                                size = content_length;
                                cache.setSize(content_length);
                            }
                        }
                    }
                    if (line.startsWith("Content-Range:")) {
                        String h[] = line.split(":\\s*");
                        //Content-Range: bytes 7147242-11625760/11625761
                        if (h.length>=2) {
                            h = h[1].split("[\\s\\-/]+");
                            if (h.length>1) {
                                start = Integer.parseInt(h[1]);
                            }
                            if (h.length>3) {
                                size = Integer.parseInt(h[3]);
                                cache.setSize(size);
                            }
                        }
                    }
                    if (line.startsWith("Content-Type:")) {
                        String h[] = line.split(":\\s*");
                        type = h[1];
                    }
                }
                Log.d("nicop_SmileVideoProxy","response ok " + type + " " + size);
                
                cache.type = type;
                
                if (!"video/mp4".equals(type)) {
                    sockout.close();
                    sockin.close();
                    soc.close();
                    return;
                }
                
                // ]
                byte buffer[] = new byte[8192];
                int len,pos=start;
                while(pos<size && (len = sockin.read(buffer))>0){
                    if (pos+len>size) len=size-pos;
                    cache.write(buffer, pos, len);
                    pos+=len;
                    Log.d("nicop_SmileVideoProxy","download " + len);
                    sleep(0);
                }
                Log.d("nicop_SmileVideoProxy","smilevideo end ");

                sockout.close();
                sockin.close();
                soc.close();
                sleep(0);
                
            } catch (InterruptedException e) {  
                
            } catch (Exception e) {  
                e.printStackTrace();
            }
        }
    }
    

    class ConnectionThread extends Thread {
        Socket client;
        int counter;
  
        public ConnectionThread(Socket client, int c) {
            this.client = client;
            this.counter = c;
        }

        // vLV
        public void run() {
            
            
            BufferedOutputStream os = null;
            BufferedInputStream sockin = null;
            BufferedWriter sockout = null;
            try {
                //@NCAg@IP
                String destIP = client.getInetAddress().toString();
                // NCAg@|[g  
                int destport = client.getPort();

                Log.d("nicoplayer_SmileVideoProxy","accept Connection " + counter + " Fconnected from "  
                        + destIP + " with port " + destport + ".");

                if (!destIP.equals("/127.0.0.1")) {
                    Log.d("nicoplayer_SmileVideoProxy"," not local IP.");
                    return;
                }

                os = new BufferedOutputStream(client.getOutputStream());
                BufferedReader reader = new BufferedReader(new InputStreamReader(client.getInputStream()));

                // NGXg
                String inline = reader.readLine();
                String request[] = inline.split(" ");
                if (request.length < 2 || !request[1].substring(0,7).equals("/video/")) {
                    Log.d("nicoplayer_SmileVideoProxy","invalid url");
                    return;
                }
                if (cache != null && cache.getCacheSize()>0) {
                    sendfile(os,reader);
                    return;
                }

                String url = URLDecoder.decode(request[1].substring(7));
                String request2[] = URLDecoder.decode(request[1].substring(7)).split("#");
                url = request2[0];
                String cookie = request2[1];
                // url = "http://nicogame.jp/test.mp4";
                Log.d("nicop","smilevideo connect ok " + url);
                

                // ڑ
                Uri uri = Uri.parse(url);
                Socket soc = new Socket(uri.getHost(),uri.getPort()<0?80:uri.getPort());
                Log.d("nicoplayer_SmileVideoProxy","socket ok");

                sockin = new BufferedInputStream(soc.getInputStream());
                sockout = new BufferedWriter(new OutputStreamWriter(soc.getOutputStream()));
                sockout.write(request[0]+" " + uri.getPath()+"?"+uri.getQuery() + " HTTP/1.1\r\n");


                Log.d("nicoplayer_SmileVideoProxy","Header recv start ");
                
                while (reader.ready() && inline != null) {                  
                    inline = reader.readLine();
                    Log.d("nicop_SmileVideoProxy","recv: "+inline);
                    if (inline.length()>10 &&  inline.substring(0,5).equals("Host:")){
                        Log.d("nicop_SmileVideoProxy","cookie: "+cookie );
                        sockout.write("Host:" + uri.getHost() + "\r\n");
                        sockout.write("Cookie:" + cookie + "\r\n");
                        continue;
                    }
                    sockout.write(inline + "\r\n");
                    if (inline.length()==0) {
                        break;
                    }
                }
                sockout.write("\r\n");
                sockout.flush();
                Log.d("nicop_SmileVideoProxy","Header recv ok ");
                
                
                // ]
                byte buf[] = new byte[8192];
                int len;
                while((len = sockin.read(buf))>0){
                    //Log.d("nicop_SmileVideoProxy","recv:"+len);
                    os.write(buf,0,len);
                }
                Log.d("nicop_SmileVideoProxy","smilevideo end ");

                os.close();
                sockout.close();
                sockin.close();
                soc.close();
            } catch (Exception e) {  
                try{
                    if (sockout!=null) sockout.close();
                    if (sockin!=null) sockin.close();
                    if (os!=null) os.close();
                } catch (Exception ee) {  
                }
                for (int i=0;i<e.getStackTrace().length;i++) {
                    Log.d("nicop_SmileVideoProxy","Exception:"+e.getStackTrace()[i]);
                }
            }
            Log.d("nicop_SmileVideoProxy","connection end");
        }
        
        // SLbVς݂̃t@C (mp4)
        private void sendfile(BufferedOutputStream output,BufferedReader input) {
            Log.d("nicop_SmileVideoProxy","sendfile");
            
            try {
                int size=cache.getSize();
                Log.d("nicop_SmileVideoProxy","size " + size);
                
                boolean range_request = false;
                int range_start = 0;
                int range_end = size-1;
                // request header
                String line;
                while ((line = input.readLine()) != null) {                  
                    if (line.length()==0) break;
                    Log.d("nicop_SmileVideoProxy","recv: "+line);
                    if (line.startsWith("Range:")) {
                        String s[] = line.split("[:\\s,\\-=]+");
                        // Range: bytes start end
                        if (s.length>2 && s[2].length()>0) {
                            range_start = Integer.parseInt(s[2]);
                        }
                        if (s.length>3 && s[3].length()>0) {
                            range_end = Integer.parseInt(s[3]);
                        }
                        range_request = true;
                    }
                }
                
                
                // response
                if (range_request) {
                    output.write("HTTP/1.1 206 Partial content\r\n".getBytes());
                    output.write(("Content-Range: bytes " + range_start+"-"+range_end+"/"+(range_end-range_start+1)+"\r\n").getBytes());
                } else {
                    output.write("HTTP/1.1 200 OK\r\n".getBytes());
                }
                output.write("Content-Type: video/mp4\r\n".getBytes());
                output.write(("Content-Length: " + size + " \r\n").getBytes());
                output.write("Accept-Ranges: bytes\r\n".getBytes());
                output.write("\r\n".getBytes());
    
                int rest = range_end-range_start+1;
                byte buffer[] = new byte[8192];
                int pos = range_start;
                int len;
                while (rest>0 && (len=cache.read(buffer,pos,buffer.length<rest?buffer.length:rest)) > 0) {
                    pos += len;
                    rest -= len;
                    output.write(buffer,0,len);
                }
            
            } catch(Exception e) {
            }
            Log.d("nicop_SmileVideoProxy","end");

        }
        
    }
}
